mouse = class("mouse")

local currentMouse = objc.GCMouse.current

local mouseInput = currentMouse.mouseInput


function mouse:init(shouldHide)
    --Pos
    self.class.x = 0
    self.class.y = 0
    self.class.inRegion = false
    self.class.lock = false
    self.class.dx = 0
    self.class.dy = 0
    self.class.active = false
    self.class.position = vec2(0)
    hoverHandler = HoverHandler()
    local hoverGesture = objc.UIHoverGestureRecognizer:alloc():initWithTarget_action_(hoverHandler, objc.selector("hovering"))
    objc.viewer.view:addGestureRecognizer_(hoverGesture)
    
    -- Hide
    delegate = PointerInteraction()
    self.class.interaction = objc.UIPointerInteraction:alloc():initWithDelegate_(delegate)
    
    self.class.interaction.delegate = delegate

end

mouse.hide = function()
    objc.viewer.view :addInteraction_(mouse.interaction)
end

mouse.show = function()
    objc.viewer.view :removeInteraction_(mouse.interaction)
end



HoverHandler = objc.class("HoverHandler")

function HoverHandler:hovering(obj)
    hoverPos = obj:locationInView_(obj.view)
    mouse.x = hoverPos.x
    mouse.y = HEIGHT-hoverPos.y
end

PointerInteraction = objc.delegate("UIPointerInteractionDelegate")

function PointerInteraction:pointerInteraction_regionForRequest_defaultRegion_(objInteraction, objRegion, objDefault)    
    local frame = objc.viewer.view.frame
    mouse.inRegion = true
    return objc.UIPointerRegion:regionWithRect_identifier_(frame, "ViewerRegion")
    
    --return objDefault
end

function PointerInteraction:pointerInteraction_styleForRegion_(objInteraction, objRegion)
    local hidden = objc.UIPointerStyle.hiddenPointerStyle
    return hidden
end

mouseInput.mouseMovedHandler = function(objMouse, floatDeltaX, floatDeltaY)
    mouse.dx = floatDeltaX
    mouse.dy = floatDeltaY
end

function touched(touch)
    mouse.x = touch.x
    mouse.y = touch.y
end

NotificationHandler = objc.class("NotificationHandler")
notificationCenter = objc.NSNotificationCenter.defaultCenter

function NotificationHandler:mouseDidConnect(objNotification)
    if(mouse.connected ~= nil) then
        mouse.connected()
    end
end

handler = NotificationHandler()

notificationCenter:removeObserver_name_object_("GCMouseDidConnectNotification")

notificationCenter:addObserver_selector_name_object_(
handler,
objc.selector("mouseDidConnect"),
"GCMouseDidConnectNotification"
)

function NotificationHandler:mouseDidDisconnect(objNotification)
    if(mouse.disconnected ~= nil) then
        mouse.disconnected()
    end
end

handler2 = NotificationHandler()

notificationCenter:addObserver_selector_name_object_(
handler2,
objc.selector("mouseDidDisconnect"),
"GCMouseDidDisconnectNotification"
)

function shutdown()
    notificationCenter:removeObserver_(handler)
    notificationCenter:removeObserver_(handler2)
end


mouse.left = {pressed = false, released = false}
mouse.right = {pressed = false, released = false}
mouse.middle = {pressed = false, released = false}
mouse.scroll = vec2(0)

local justChanged = false

mouseInput.leftButton.pressedChangedHandler = function(objButton, floatValue, boolPressed)
    if boolPressed then
        mouse.left.pressed = true
    else
        mouse.left.released = true
    end
    justChanged = true
    
end

mouseInput.rightButton.pressedChangedHandler = function(objButton, floatValue, boolPressed)
    if boolPressed then
        mouse.right.pressed = true
    else
        mouse.right.released = true
    end
    justChanged = true
    
end

mouseInput.middleButton.pressedChangedHandler = function(objButton, floatValue, boolPressed)
    if boolPressed then
        mouse.middle.pressed = true
    else
        mouse.middle.released = true
    end
    justChanged = true
    
end

function update()
    mouse.scroll.x = mouseInput.scroll.xAxis.value
    mouse.scroll.y = mouseInput.scroll.yAxis.value
    mouse.left.pressing = mouseInput.leftButton.value == 1.0
    mouse.left.value = mouseInput.leftButton.value
    mouse.right.pressing = mouseInput.rightButton.value == 1.0
    mouse.right.value = mouseInput.rightButton.value
    mouse.middle.pressing = mouseInput.middleButton.value == 1.0
    mouse.middle.value = mouseInput.middleButton.value 
    mouse.position = vec2(mouse.x, mouse.y)
    viewer.pointerLocked = mouse.lock
    if not justChanged then
        mouse.left.pressed = false
        mouse.left.released = false
        mouse.right.pressed = false
        mouse.right.released = false
        mouse.middle.pressed = false
        mouse.middle.released = false
    else
        justChanged = false
    end

    if(mouse.x == 0 and mouse.y == HEIGHT) and not mouse.lock then
        mouse.inRegion = false
    else
        mouse.inRegion = true
    end
    mouse.active = (objc.GCMouse.current ~= nil)
end
mouse()